@@ -86,7 +86,9 @@ on_print_menu_activate                 (GtkMenuItem     *menuitem,
 {
    image_buf *ibuf = widget_get_image(GTK_WIDGET(menuitem));
    
+   BEGIN_BUSY_CURSOR
    do_print(ibuf);
+   END_BUSY_CURSOR
    
 
 
@@ -170,7 +172,12 @@ void
 on_flip_x_axis_menu_activate           (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-
+   image_buf *ibuf = widget_get_image(GTK_WIDGET(menuitem));
+   BEGIN_BUSY_CURSOR
+   image_buf_force_select_region(ibuf); /* for now */
+   
+   image_buf_flip_x_region(ibuf);
+   END_BUSY_CURSOR
 }
 
 
@@ -178,6 +185,12 @@ void
 on_flip_y_axis_menu_activate           (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+   image_buf *ibuf = widget_get_image(GTK_WIDGET(menuitem));
+   BEGIN_BUSY_CURSOR
+   image_buf_force_select_region(ibuf); /* for now */
+   
+   image_buf_flip_y_region(ibuf);
+   END_BUSY_CURSOR
 
 }
 
@@ -186,6 +199,9 @@ void
 on_rotate_p90_menu_activate            (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+   image_buf *ibuf = widget_get_image(GTK_WIDGET(menuitem));
+   image_buf_force_select_region(ibuf); /* for now */
+   image_buf_rotate_region(ibuf, 90 * M_PI / 180);
 
 }
 
@@ -194,6 +210,9 @@ void
 on_rotate_p45_menu_activate            (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+   image_buf *ibuf = widget_get_image(GTK_WIDGET(menuitem));
+   image_buf_force_select_region(ibuf); /* for now */
+   image_buf_rotate_region(ibuf, 45 * M_PI / 180);
 
 }
 
@@ -202,6 +221,9 @@ void
 on_rotate_n45_menu_activate            (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+   image_buf *ibuf = widget_get_image(GTK_WIDGET(menuitem));
+   image_buf_force_select_region(ibuf); /* for now */
+   image_buf_rotate_region(ibuf, -45 * M_PI / 180);
 
 }
 
@@ -210,6 +232,10 @@ void
 on_rotate_n90_menu_activate            (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+   image_buf *ibuf = widget_get_image(GTK_WIDGET(menuitem));
+   image_buf_force_select_region(ibuf); /* for now */
+   image_buf_rotate_region(ibuf, -90 * M_PI / 180);
+
 
 }
 
@@ -218,6 +244,9 @@ void
 on_rotate_p180_menu_activate           (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+   image_buf *ibuf = widget_get_image(GTK_WIDGET(menuitem));
+   image_buf_force_select_region(ibuf); /* for now */
+   image_buf_rotate_region(ibuf, 180 * M_PI / 180);
 
 }
 
@@ -226,6 +255,9 @@ void
 on_rotate_menu_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
+   image_buf *ibuf = widget_get_image(GTK_WIDGET(menuitem));
+   image_buf_force_select_region(ibuf); /* for now */
+   image_buf_rotate_region(ibuf, 60 * M_PI / 180);
 
 }
 
@@ -1252,6 +1284,7 @@ on_line_width_combo_combo_entry_changed
    g_assert(t != 0);
    gdk_gc_set_line_attributes(ibuf->gc, t, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);
    g_free(tmp);
+   gtk_widget_grab_focus(ibuf->drawing_area); /* force the line width entry widget to give up focus */
 }
 
 
@@ -1729,3 +1762,42 @@ on_get_desktop_activate                (GtkMenuItem     *menuitem,
 
 }
 
+
+void
+on_undo_menu_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+
+}
+
+
+void
+on_about_gnome_activate                (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    gnome_url_show("http://www.gnome.org/");
+}
+
+
+void
+on_about_gnu_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    gnome_url_show("http://www.gnu.org/");
+
+}
+
+
+gboolean
+on_about_dialog_button_release_event   (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data)
+{
+    gnome_url_show("http://www.gnu.org/");
+
+
+    return FALSE;
+}
+
+
+
