@@ -766,3 +766,25 @@ on_print_preview_activate              (GtkMenuItem     *menuitem,
 void
 on_get_desktop_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+
+void
+on_undo_menu_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_about_gnome_activate                (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_about_gnu_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+gboolean
+on_about_dialog_button_release_event   (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_about_dialog_button_release_event   (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
