@@ -1,6 +1,6 @@
 
 /*
-    Copyright 2000  Li-Cheng (Andy) Tai
+    Copyright 2000  Li-Cheng (Andy) Tai (atai@gnu.org)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -81,6 +81,8 @@ static void clear_image_buf(image_buf *ibuf)
 {
    if (ibuf->rgbbuf)
       gdk_pixbuf_unref(ibuf->rgbbuf);
+   if (ibuf->regionbuf)
+      gdk_pixbuf_unref(ibuf->regionbuf);
    if (ibuf->pixmap)
       gdk_pixmap_unref(ibuf->pixmap);
    if (ibuf->gc)
@@ -93,6 +95,7 @@ static void clear_image_buf(image_buf *ibuf)
       free(ibuf->name);
   
    ibuf->rgbbuf = 0;
+   ibuf->regionbuf = 0;
    ibuf->pixmap = 0;
    ibuf->gc = 0;
    ibuf->name = 0;
@@ -269,7 +272,6 @@ void image_buf_process_in_place(image_buf *ibuf, void (*f)(image_buf *, image_bu
    f(ibuf, &tmpbuf);
    gdk_pixbuf_unref(ibuf->rgbbuf);   
    ibuf->rgbbuf = tmpbuf.rgbbuf;
-   ibuf->modified = 1;
    image_buf_rgbbuf_to_pixmap(ibuf, 0);
 }
    
@@ -319,11 +321,12 @@ void image_buf_pixmap_to_rgbbuf(image_buf *ibuf, GdkRectangle *irect)
    fullrect.width = image_buf_width(ibuf);
    fullrect.height = image_buf_height(ibuf);
    
-   if (irect)
+   if (!ibuf->modified && irect)
       gdk_rectangle_intersect(&fullrect, irect, &rect);
    else
       rect = fullrect;
    gdk_pixbuf_get_from_drawable(ibuf->rgbbuf, ibuf->pixmap, gdk_rgb_get_cmap(), rect.x, rect.y, rect.x, rect.y, rect.width, rect.height);
+   ibuf->modified = 0;
 }
 
 void image_buf_rgbbuf_to_pixmap(image_buf *ibuf, GdkRectangle *irect)
@@ -576,3 +579,146 @@ void image_buf_set_pts_top_left(image_buf * ibuf, int x, int y)
    }
 }
 
+
+/* taken from gd.c in gd 1.8.3 */
+static int cmp_int(const void *a, const void *b)
+{
+	return (*(const int *)a) - (*(const int *)b);
+}
+
+static void fill_polygon(unsigned char *data, int pixelsize, int rowstride, unsigned char value[4], unsigned char mask[4], GdkPoint *pts, int num_pts, int xoffset, int yoffset)
+{
+   /* this assumes the data is in RGBA format, each channel takes 1 bytes (thus 4 bytes per pixel) */
+
+   int x;
+   int j;
+   unsigned char *c;
+   
+/* taken from gd.c in gd 1.8.3 */
+   
+   int i;
+   int y;
+   int miny, maxy;
+   int x1, y1;
+   int x2, y2;
+   int ind1, ind2;
+   int ints;
+   int * polyInts = 0;
+   int polyAllocated = 0;
+   
+   assert(xoffset >= 0);
+   assert(yoffset >= 0);
+   
+   if (!num_pts) {
+	   return;
+   }
+   if (!polyAllocated) {
+	   polyInts = (int *) malloc(sizeof(int) * num_pts);
+	   polyAllocated = num_pts;
+   }		
+
+   miny = pts[0].y;
+   maxy = pts[0].y;
+   for (i=1; (i < num_pts); i++) {
+	   if (pts[i].y < miny) {
+		   miny = pts[i].y;
+	   }
+	   if (pts[i].y > maxy) {
+		   maxy = pts[i].y;
+	   }
+   }
+   /* Fix in 1.3: count a vertex only once */
+   for (y=miny; (y <= maxy); y++) {
+/*1.4		int interLast = 0; */
+/*		int dirLast = 0; */
+/*		int interFirst = 1; */
+	   ints = 0;
+	   for (i=0; (i < num_pts); i++) {
+		   if (!i) {
+			   ind1 = num_pts-1;
+			   ind2 = 0;
+		   } else {
+			   ind1 = i-1;
+			   ind2 = i;
+		   }
+		   y1 = pts[ind1].y;
+		   y2 = pts[ind2].y;
+		   if (y1 < y2) {
+			   x1 = pts[ind1].x;
+			   x2 = pts[ind2].x;
+		   } else if (y1 > y2) {
+			   y2 = pts[ind1].y;
+			   y1 = pts[ind2].y;
+			   x2 = pts[ind1].x;
+			   x1 = pts[ind2].x;
+		   } else {
+			   continue;
+		   }
+		   if ((y >= y1) && (y < y2)) {
+			   polyInts[ints++] = (y-y1) * (x2-x1) / (y2-y1) + x1;
+		   } else if ((y == maxy) && (y > y1) && (y <= y2)) {
+			   polyInts[ints++] = (y-y1) * (x2-x1) / (y2-y1) + x1;
+		   }
+	   }
+	   qsort(polyInts, ints, sizeof(int), cmp_int);
+
+	   for (i=0; (i < (ints)); i+=2) {
+	      c = data + (y - yoffset) * rowstride + (polyInts[i] - xoffset) * pixelsize;
+	      for (x = polyInts[i]; x <= polyInts[i+1] ; x++)
+	      {
+	         for (j = 0; j < pixelsize; j++, c++)
+		 {
+		    if (mask[j])
+		       *c = value[j];
+		 }
+                 
+	      }
+	      
+	   }
+   }
+   if (polyInts)
+      free(polyInts);
+}
+   
+void image_buf_select_current_region(image_buf *ibuf)
+{
+   if (ibuf->num_pts == 0)
+   {
+       return; /* nothing to select */
+   }
+   image_buf_get_selected_region_to_pixbuf(ibuf, & ibuf->regionbuf);
+   
+}
+	 
+void image_buf_get_selected_region_to_pixbuf(image_buf *ibuf, GdkPixbuf **pixbuf) /* copy the selected region of the image into a GdkPixbuf */
+{
+   GdkRectangle rect;
+   unsigned char alphaonly[4] = {0, 0, 0, 1};
+   unsigned char *data = 0, *c, *d;
+   unsigned char default_pixel[4] = {0, 0, 0, 255};
+   int x, y, i;
+   int pixel_size;
+   rect = compute_cover_rect(ibuf->pts, ibuf->num_pts);
+   
+   image_buf_pixmap_to_rgbbuf(ibuf, &rect);
+   
+   if (*pixbuf)
+      gdk_pixbuf_unref(*pixbuf);
+   *pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, rect.width, rect.height);
+   assert(*pixbuf);
+   pixel_size = gdk_pixbuf_get_n_channels(*pixbuf) * gdk_pixbuf_get_bits_per_sample(*pixbuf) / 8;
+   
+   gdk_pixbuf_copy_area(ibuf->rgbbuf, rect.x, rect.y, rect.width, rect.height, *pixbuf, 0, 0);
+   data = gdk_pixbuf_get_pixels( *pixbuf);
+   for (y = 0, c = data; y < rect.height; y++, c += gdk_pixbuf_get_rowstride( *pixbuf))
+   {
+      d = c;
+      for (x = 0; x < rect.width; x++, d += pixel_size)
+      {
+         *(d + 3) = 0;/* default everything is transparent */
+      }
+   }
+   
+   fill_polygon(data, pixel_size, gdk_pixbuf_get_rowstride( *pixbuf), default_pixel, alphaonly, ibuf->pts, ibuf->num_pts, rect.x, rect.y);
+}
+    
