@@ -37,12 +37,17 @@ DRAWING_TOOL ;
 #define NUM_PALETTE_ENTRIES (14 * 2)
 
 
-#define IMAGE_MODIFIED image_buf_set_modified(ibuf, 1) 
+#define IMAGE_MODIFIED {\
+       image_buf_set_modified(ibuf, 1);\
+       /*image_buf_pixmap_to_rgbbuf(ibuf, 0);*/\
+    }
+      
+
 
 
 typedef struct image_buf
 {
-   GdkPixbuf *rgbbuf; 
+   GdkPixbuf *rgbbuf, *regionbuf; 
    GdkPixmap *pixmap; 
    GdkGC *gc;
    GtkWidget *window;
@@ -58,6 +63,8 @@ typedef struct image_buf
    
    GdkPoint pts[MAX_NUM_POINTS]; /* a temporary array for storing points */
    int num_pts;
+   int ptdiffx;
+   int ptdiffy;
    
    int modified;
    int has_focus;
@@ -116,7 +123,8 @@ void image_buf_focus_lost(image_buf*);
 int image_buf_has_focus(image_buf*);
 
 void image_buf_set_pts_top_left(image_buf * ibuf, int x, int y);
-
+void image_buf_get_selected_region_to_pixbuf(image_buf *ibuf, GdkPixbuf **pixbuf); /* copy the selected region of the image into a GdkPixbuf */
+void image_buf_select_current_region(image_buf *ibuf);
 
 
 #endif
