@@ -1,6 +1,6 @@
 
 /*
-    Copyright 2000  Li-Cheng (Andy) Tai
+    Copyright 2000, 2001  Li-Cheng (Andy) Tai (atai@gnu.org)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <assert.h>
 #include <sys/param.h>
 #include <gdk/gdkx.h>
+#include <math.h>
 #include "util.h"
 #include "support.h"
 #include "ui.h"
@@ -33,8 +34,35 @@
 GdkCursor *busy_cursor = 0;
 GdkCursor *arrow_cursor = 0;
 
+
+void image_buf_force_select_region(image_buf *ibuf)
+{
+   DRAWING_TOOL tool = ibuf->current_tool;
+   if (tool == PASTE)
+   {
+       image_buf_copy_clipboard_to_region_buf(ibuf);
+       return;
+   }
+   
+   if (ibuf->num_pts == 0)
+   {
+       /* no points selected, then let's treat this as if the whole image is selected */
+       image_buf_select_all(ibuf);
+       image_buf_select_current_region(ibuf);
+       ibuf->lpx = ibuf->lpy = 0;
+       assert(ibuf->num_pts == 4); /* an image must be a rectangle, thus 4 points */
+   }
+   else
+   {
+       image_buf_select_current_region(ibuf); /* force currently selected region to be selected, regardless of the original content */
+   }
+}    
+
+
 const int erase_size = 4;
 
+
+
 GtkWidget *widget_get_toplevel_parent(GtkWidget *widget)
 {
    GtkWidget *parent, *found_widget;
@@ -56,6 +84,8 @@ GtkWidget *widget_get_toplevel_parent(GtkWidget *widget)
    return found_widget;
 }
 
+
+
 image_buf *widget_get_image(GtkWidget *widget)
 {
    image_buf *ibuf = (image_buf*) gtk_object_get_user_data(GTK_OBJECT(widget));
@@ -302,6 +332,11 @@ void set_drawing_tool(image_buf *ibuf, DRAWING_TOOL tool)
    const char *buttonname = 0;
    GtkWidget *selected = 0;
    
+   if (ibuf->modified)
+   {
+      image_buf_pixmap_to_rgbbuf(ibuf, 0);
+   }
+   
    if (tool < NONE)
       buttonname = toggle_button_infos[tool - ERASE].name;
    
@@ -696,7 +731,8 @@ void image_buf_clear(image_buf *ibuf)
 
    draw_rectangle(ibuf->pixmap, ibuf->gc, TRUE, clientrect.x, clientrect.y, clientrect.width, clientrect.height);
    draw_rectangle(ibuf->drawing_area->window, ibuf->gc, TRUE, clientrect.x, clientrect.y, clientrect.width, clientrect.height);
-   IMAGE_MODIFIED;
+   image_buf_pixmap_to_rgbbuf(ibuf, 0);
+
    
    gdk_gc_set_foreground(ibuf->gc, &(gcvalues.foreground));
 
@@ -811,7 +847,17 @@ void handle_button_press(image_buf *ibuf, int x, int y)
       	 
        
    case PASTE:
-      image_buf_put_clipboard_image(ibuf, x, y);
+      rect = compute_cover_rect(ibuf->pts, ibuf->num_pts);
+      ibuf->ptdiffx = rect.x - x;
+      ibuf->ptdiffy = rect.y - y;
+      if ((ibuf->ptdiffx > 0) || (ibuf->ptdiffx <= -rect.width) || 
+          (ibuf->ptdiffy > 0) || (ibuf->ptdiffy <= -rect.height))
+      {
+	 image_buf_leave_paste_mode(ibuf);
+         return;
+      }      
+      image_buf_adjust_according_to_points(ibuf, &x, &y);
+      image_buf_put_region_image(ibuf, x, y);
       break;
                 
    default:
@@ -937,7 +983,8 @@ void handle_button_move(image_buf *ibuf, int x, int y)
        
        
    case PASTE:
-      image_buf_put_clipboard_image(ibuf, x, y);
+      image_buf_adjust_according_to_points(ibuf, &x, &y);
+      image_buf_put_region_image(ibuf, x, y);
       break;
                 
    default:
@@ -1088,7 +1135,8 @@ void handle_button_release(image_buf *ibuf, int x, int y)
    
    
    case PASTE:
-      image_buf_put_clipboard_image(ibuf, x, y);
+      image_buf_adjust_according_to_points(ibuf, &x, &y);
+      image_buf_put_region_image(ibuf, x, y);
       break;
          
    default:
@@ -1132,15 +1180,19 @@ void handle_key_release(image_buf *ibuf, GdkEventKey *keyevent)
    int length;
    char tmp[MAX_TEXT_LENGTH];
    
-   if ((ibuf->current_tool == PASTE))
+   if (ibuf->current_tool == PASTE)
    {
       if (keyevent->keyval == GDK_Escape)
       {
          image_buf_clear_flash(ibuf);
-	 image_buf_erase_dynamic_image(ibuf, clipboard, ibuf->lpx, ibuf->lpy);
+	 image_buf_erase_dynamic_image(ibuf, ibuf->regionbuf, ibuf->lpx, ibuf->lpy);
 	 image_buf_leave_paste_mode(ibuf);
 	 return;
       }
+      else if (keyevent->keyval == GDK_Return)
+      {
+         image_buf_pixmap_to_rgbbuf(ibuf, 0); /* make what's in the pixmap into the rgb buffer */
+      }
    }
       
    
@@ -1251,6 +1303,10 @@ void image_buf_restore_tool(image_buf *ibuf)
 
 void image_buf_enter_paste_mode(image_buf *ibuf)
 {
+   if (ibuf->modified)
+   {
+      image_buf_pixmap_to_rgbbuf(ibuf, 0);
+   }
    image_buf_save_tool(ibuf);
    set_drawing_tool(ibuf, PASTE);
 }
@@ -1258,7 +1314,6 @@ void image_buf_enter_paste_mode(image_buf *ibuf)
 void image_buf_leave_paste_mode(image_buf *ibuf)
 {
    image_buf_pixmap_to_rgbbuf(ibuf, 0);
-   IMAGE_MODIFIED;
    image_buf_restore_tool(ibuf);
 }
 
@@ -1285,11 +1340,13 @@ void image_buf_draw_dynamic_image(image_buf *ibuf, GdkPixbuf *new_image, int x,
    if (y + rect.height >= image_buf_height(ibuf))
       rect.height = image_buf_height(ibuf) - y - 1;
    
+   image_buf_clear_flash(ibuf);
+   
    if ((image_buf_width(ibuf) - x - 1 < 0) || (image_buf_height(ibuf) - y - 1 < 0))
       return;
       
-   image_buf_clear_flash(ibuf);
-      
+   if ((x + rect.width < 0) || (y + rect.height < 0))   
+      return;
    
    image_buf_rgbbuf_to_pixmap(ibuf, &rect);
    
@@ -1320,23 +1377,25 @@ void image_buf_erase_dynamic_image(image_buf *ibuf, GdkPixbuf *new_image, int x,
    
    if (y + rect.height >= image_buf_height(ibuf))
       rect.height = image_buf_height(ibuf) - y - 1;
+      
+   image_buf_clear_flash(ibuf);
    
    if ((image_buf_width(ibuf) - x - 1 < 0) || (image_buf_height(ibuf) - y - 1 < 0))
       return;
-      
-   image_buf_clear_flash(ibuf);
+   if ((x + rect.width < 0) || (y + rect.height < 0))   
+      return;
    
    image_buf_rgbbuf_to_pixmap(ibuf, &rect);
    gtk_widget_queue_draw_area(ibuf->drawing_area, rect.x, rect.y, rect.width, rect.height);
 }   
 
-void image_buf_put_clipboard_image(image_buf *ibuf, int x, int y)
+void image_buf_put_region_image(image_buf *ibuf, int x, int y)
 {
    if ((ibuf->lpx != INT_MIN) && (ibuf->lpy != INT_MIN))
    {
-      image_buf_erase_dynamic_image(ibuf, clipboard, ibuf->lpx, ibuf->lpy);
+      image_buf_erase_dynamic_image(ibuf, ibuf->regionbuf, ibuf->lpx, ibuf->lpy);
    }
-   image_buf_draw_dynamic_image(ibuf, clipboard, x, y);
+   image_buf_draw_dynamic_image(ibuf, ibuf->regionbuf, x, y);
    image_buf_set_pts_top_left(ibuf, x, y); 
    ibuf->lpx = x;
    ibuf->lpy = y;   
@@ -1344,6 +1403,82 @@ void image_buf_put_clipboard_image(image_buf *ibuf, int x, int y)
 
 }   
 
+void image_buf_flip_x_region(image_buf *ibuf)
+{
+   if ((ibuf->lpx != INT_MIN) && (ibuf->lpy != INT_MIN))
+   {
+      image_buf_erase_dynamic_image(ibuf, ibuf->regionbuf, ibuf->lpx, ibuf->lpy);
+      pts_flip_x(ibuf->pts, ibuf->num_pts);
+      pixbuf_flip_x(& ibuf->regionbuf);
+      image_buf_draw_dynamic_image(ibuf, ibuf->regionbuf, ibuf->lpx, ibuf->lpy);
+      if (ibuf->current_tool != PASTE)
+      {
+         image_buf_pixmap_to_rgbbuf(ibuf, 0);
+      }
+   }
+   
+
+}
+
+void image_buf_flip_y_region(image_buf *ibuf)
+{
+   if ((ibuf->lpx != INT_MIN) && (ibuf->lpy != INT_MIN))
+   {
+      image_buf_erase_dynamic_image(ibuf, ibuf->regionbuf, ibuf->lpx, ibuf->lpy);
+      pts_flip_y(ibuf->pts, ibuf->num_pts);
+      pixbuf_flip_y(& ibuf->regionbuf);
+      image_buf_draw_dynamic_image(ibuf, ibuf->regionbuf, ibuf->lpx, ibuf->lpy);
+      if (ibuf->current_tool != PASTE)
+      {
+         image_buf_pixmap_to_rgbbuf(ibuf, 0);
+      }
+      
+   }
+
+}
+
+
+void image_buf_rotate_region(image_buf *ibuf, double rad)
+{
+  // rotate the regionbuf by rad radians
+   double m[2][2];
+   int mx, my;
+   GdkRectangle rect;
+
+   m[0][0] = cos(rad);
+   m[0][1] = -sin(rad);
+   m[1][0] = sin(rad);
+   m[1][1] = cos(rad);
+   
+   rect = compute_cover_rect(ibuf->pts, ibuf->num_pts);
+   mx = rect.x + rect.width / 2;
+   my = rect.y + rect.height / 2;
+   
+   image_buf_erase_dynamic_image(ibuf, 0, ibuf->lpx, ibuf->lpy);
+   image_buf_set_pts_top_left(ibuf, -mx, -my);
+   
+   
+   pixbuf_apply_matrix22(& ibuf->regionbuf, m);
+   pts_apply_matrix22(ibuf->pts, ibuf->num_pts, m);
+   rect = compute_cover_rect(ibuf->pts, ibuf->num_pts);
+   
+   ibuf->lpx = mx - rect.width / 2;
+   ibuf->lpy = my - rect.height / 2;
+   
+   image_buf_set_pts_top_left(ibuf, ibuf->lpx, ibuf->lpy);
+   
+   image_buf_draw_dynamic_image(ibuf, ibuf->regionbuf, ibuf->lpx, ibuf->lpy);
+      
+   if (ibuf->current_tool != PASTE)
+   {
+      image_buf_pixmap_to_rgbbuf(ibuf, 0);
+   }
+   
+  
+}
+
+
+
 
 void image_buf_get_desktop(image_buf *ibuf)
 {
@@ -1466,3 +1601,230 @@ GdkRectangle compute_cover_rect(GdkPoint *pts, int num_pts)
    return rect;
 }
    
+void image_buf_adjust_according_to_points(image_buf *ibuf, int *x, int *y)
+{
+   
+   *x += ibuf->ptdiffx;
+   *y += ibuf->ptdiffy;
+}
+
+void pts_flip_x(GdkPoint *pts, int num_pts)
+{
+   GdkRectangle rect = compute_cover_rect(pts, num_pts);
+   int i;
+   int x = rect.x + ((rect.width)) / 2;
+   if (rect.width & 1)
+   {  /* odd */
+      for (i = 0; i < num_pts; i++)
+      {
+	 pts[i].x = (int)(2 * x - pts[i].x);
+      }
+   }
+   else
+   {  /* even */
+      for (i = 0; i < num_pts; i++)
+      {
+	 pts[i].x = (int)(2 * x - pts[i].x - 1);
+      }
+   }
+
+}
+
+void pts_flip_y(GdkPoint *pts, int num_pts)
+{
+   GdkRectangle rect = compute_cover_rect(pts, num_pts);
+   int i;
+   double  y = rect.y + ((rect.height)) / 2;
+   if (rect.height & 1)
+   {  /* odd */
+      for (i = 0; i < num_pts; i++)
+      {
+	 pts[i].y = (int)(2 * y - pts[i].y);
+      }
+   }
+   else
+   {  /* even */
+      for (i = 0; i < num_pts; i++)
+      {
+	 pts[i].y = (int)(2 * y - pts[i].y - 1);
+      }
+   }
+}
+
+void pixbuf_flip_x(GdkPixbuf** pixbuf)
+{
+
+   int width, height;
+   GdkPixbuf *newpixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, gdk_pixbuf_get_bits_per_sample(*pixbuf), width = gdk_pixbuf_get_width(*pixbuf), height = gdk_pixbuf_get_height(*pixbuf));
+   int rowstride = gdk_pixbuf_get_rowstride(newpixbuf);
+   int x, y;
+   int j;
+   unsigned char *p, *p2, *p3, *p4;
+   int pixel_size = gdk_pixbuf_get_bits_per_sample(*pixbuf) * gdk_pixbuf_get_n_channels(*pixbuf) / 8;
+
+   g_assert(newpixbuf);
+   for (p = gdk_pixbuf_get_pixels(newpixbuf), p2 = gdk_pixbuf_get_pixels(*pixbuf), y = 0; y < height; y++, p += rowstride, p2 += rowstride)
+   {
+      p4 = p2 + pixel_size * (width - 1);
+      p3 = p;
+      for (x = 0; x < width; x++)
+      {
+	 for (j = 0; j < pixel_size; j++)
+	    *(p3 + j) = *(p4 + j);
+         p3 += pixel_size;
+	 p4 -= pixel_size;
+      }
+     
+   }
+   gdk_pixbuf_unref(*pixbuf); 
+   *pixbuf = newpixbuf;     
+}
+
+void pixbuf_flip_y(GdkPixbuf** pixbuf)
+{
+
+   int width, height;
+   GdkPixbuf *newpixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, gdk_pixbuf_get_bits_per_sample(*pixbuf), width = gdk_pixbuf_get_width(*pixbuf), height = gdk_pixbuf_get_height(*pixbuf));
+   int rowstride = gdk_pixbuf_get_rowstride(newpixbuf);
+   int x, y;
+   int j;
+   unsigned char *p, *p2, *p3, *p4;
+   int pixel_size = gdk_pixbuf_get_bits_per_sample(*pixbuf) * gdk_pixbuf_get_n_channels(*pixbuf) / 8;
+
+   g_assert(newpixbuf);
+   for (p = gdk_pixbuf_get_pixels(newpixbuf), p2 = gdk_pixbuf_get_pixels(*pixbuf) + rowstride * (height - 1), y = 0; y < height; y++, p += rowstride, p2 -= rowstride)
+   {
+      p3 = p;
+      p4 = p2;
+      for (x = 0; x < width; x++)
+      {
+	 for (j = 0; j < pixel_size; j++)
+	    *(p3 + j) = *(p4 + j);
+         p3 += pixel_size;
+	 p4 += pixel_size;
+      }
+      
+   }
+   gdk_pixbuf_unref(*pixbuf); 
+   *pixbuf = newpixbuf;     
+
+}
+
+/* the below 2d transform routine contains code 
+   taken from xpaint, with the following copyright notice */
+/* +-------------------------------------------------------------------+ */
+/* | Copyright 1992, 1993, David Koblas (koblas@netcom.com)            | */
+/* | Copyright 1995, 1996 Torsten Martinsen (bullestock@dk-online.dk)  | */
+/* |                                                                   | */
+/* | Permission to use, copy, modify, and to distribute this software  | */
+/* | and its documentation for any purpose is hereby granted without   | */
+/* | fee, provided that the above copyright notice appear in all       | */
+/* | copies and that both that copyright notice and this permission    | */
+/* | notice appear in supporting documentation.  There is no           | */
+/* | representations about the suitability of this software for        | */
+/* | any purpose.  this software is provided "as is" without express   | */
+/* | or implied warranty.                                              | */
+/* |                                                                   | */
+/* +-------------------------------------------------------------------+ */
+
+
+#define XFORM(x,y,mat,nx,ny)	nx = mat[0][0] * x + mat[0][1] * y; \
+				ny = mat[1][0] * x + mat[1][1] * y
+#define COPY_MAT(s,d)	d[0][0] = s[0][0]; d[0][1] = s[0][1]; \
+			d[1][0] = s[1][0]; d[1][1] = s[1][1]
+
+#define INVERT_MAT(mat, inv) do {			\
+		float _d = 1.0 / (mat[0][0] * mat[1][1] \
+			      - mat[0][1] * mat[1][0]);	\
+		(inv)[0][0] =  (mat)[1][1] * _d;	\
+		(inv)[1][1] =  (mat)[0][0] * _d;	\
+		(inv)[0][1] = -(mat)[0][1] * _d;	\
+		(inv)[1][0] = -(mat)[1][0] * _d;	\
+	} while (0)
+
+
+#define ROUND(x) ((int)(x + 0.5))
+
+void pixbuf_apply_matrix22(GdkPixbuf **pixbuf, double matrix[2][2])
+{
+   GdkPixbuf *newpixbuf;
+   double invmatrix[2][2];
+   int x, y, width, height, i, j;
+   int rowstride1, rowstride2, pixel_size;
+   unsigned char *p1, *p2, *p3, *p4;
+   double tx[4], ty[4], minx, miny, maxx, maxy, nx, ny, width2, height2;
+   /* find the new bounding box */
+   
+   width = gdk_pixbuf_get_width(*pixbuf);
+   height = gdk_pixbuf_get_height(*pixbuf);
+   XFORM(-width / 2, -height / 2, matrix, tx[0], ty[0]);
+   XFORM(-width / 2, height / 2, matrix, tx[1], ty[1]);
+   XFORM(width / 2, -height / 2, matrix, tx[2], ty[2]);
+   XFORM(width / 2, height / 2, matrix, tx[3], ty[3]);
+   minx = INT_MAX;
+   miny = INT_MAX;
+   maxx = -INT_MAX;
+   maxy = -INT_MAX;
+   for (i = 0; i < 4; i++)
+   {
+      if (minx > tx[i])
+         minx = tx[i];
+      if (maxx < tx[i])
+         maxx = tx[i];
+      if (miny > ty[i])
+         miny = ty[i];
+      if (maxy < ty[i])
+         maxy = ty[i];
+   }
+   width2 = maxx - minx;
+   height2 = maxy - miny;
+   
+   
+   INVERT_MAT(matrix, invmatrix);
+   
+   p1 = gdk_pixbuf_get_pixels(*pixbuf);
+   newpixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, gdk_pixbuf_get_bits_per_sample(*pixbuf), ROUND(width2), ROUND(height2));
+   g_assert(newpixbuf);
+   p2 = gdk_pixbuf_get_pixels(newpixbuf);
+   rowstride1 = gdk_pixbuf_get_rowstride(*pixbuf);
+   rowstride2 = gdk_pixbuf_get_rowstride(newpixbuf);
+   pixel_size = gdk_pixbuf_get_bits_per_sample(*pixbuf) * gdk_pixbuf_get_n_channels(*pixbuf) / 8;
+   
+#define PIXEL(x, y) (p1 + ROUND(y) * rowstride1 + ROUND(x) * pixel_size)
+   
+   for (y = miny; y < maxy; y++, p2 += rowstride2)
+   {
+      p4 = p2;
+      for (x = minx; x < maxx; x++, p4 += pixel_size)
+      {
+         XFORM(x, y, invmatrix, nx, ny);
+	 nx += width / 2;
+	 ny += height / 2;
+	 g_assert(nx >= 0);
+	 g_assert(ny >= 0);
+	 g_assert(nx < gdk_pixbuf_get_width(*pixbuf));
+	 g_assert(ny < gdk_pixbuf_get_height(*pixbuf));
+	 
+	 p3 = PIXEL(nx, ny);
+	 for (j = 0; j < pixel_size; j++)
+	    *(p4 + j) = *(p3 + j);
+      }
+   }
+   
+   gdk_pixbuf_unref(*pixbuf);      
+   
+   *pixbuf = newpixbuf;
+
+}
+
+void pts_apply_matrix22(GdkPoint *pts, int num_pts, double matrix[2][2])
+{
+   int i;
+   double nx, ny;
+   for (i = 0; i < num_pts; i++)
+   {
+      XFORM(pts[i].x, pts[i].y, matrix, nx, ny);
+      pts[i].x = ROUND(nx);
+      pts[i].y = ROUND(ny);
+   }
+}
