@@ -87,15 +87,30 @@ void image_buf_restore_tool(image_buf *);
 void image_buf_enter_paste_mode(image_buf *);
 void image_buf_leave_paste_mode(image_buf *);
 
-void image_buf_put_clipboard_image(image_buf *, int, int);
+void image_buf_put_region_image(image_buf *, int, int);
 
 void image_buf_draw_dynamic_image(image_buf *, GdkPixbuf *, int, int);
 void image_buf_erase_dynamic_image(image_buf *, GdkPixbuf *, int, int);
 
+void image_buf_flip_x_region(image_buf *);
+void image_buf_flip_y_region(image_buf *);
+void image_buf_rotate_region(image_buf *, double rad);
+
 GdkRectangle compute_cover_rect(GdkPoint *pts, int num_pts);
 
+void image_buf_adjust_according_to_points(image_buf *ibuf, int *, int *);
+
 void image_buf_get_desktop(image_buf *);
 
 void image_buf_set_desktop_background(image_buf*, int);
 
+void pts_flip_x(GdkPoint *, int);
+void pts_flip_y(GdkPoint *, int);
+void pixbuf_flip_x(GdkPixbuf**);
+void pixbuf_flip_y(GdkPixbuf**);
+void pixbuf_apply_matrix22(GdkPixbuf **pixbuf, double matrix[2][2]);
+void pts_apply_matrix22(GdkPoint *pts, int num_pts, double matrix[2][2]);
+
+void image_buf_force_select_region(image_buf *ibuf);
+
 #endif
